/*
 * Web Designer
 * Copyright (C) 1999-2000  LADISLAS Willhem
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author's email : zillprog@freesurf.fr
 * 
 * snailmail :
 * LADISLAS Willhem
 * 2, rue du Serpolet
 * 95490 VAUREAL
 * FRANCE
 */

#include "common_headers.h"

void
move_notebook_page (GtkWidget * widget, int Opcode)
{

	switch (Opcode)
	  {
	  case DOC_GOTO_PREV:
		  gtk_notebook_prev_page (GTK_NOTEBOOK (notebook));
		  break;

	  case DOC_GOTO_NEXT:
		  gtk_notebook_next_page (GTK_NOTEBOOK (notebook));
		  break;

	  case DOC_GOTO_LAST:
		  gtk_notebook_set_page (GTK_NOTEBOOK (notebook), gtk_notebook_page_num (GTK_NOTEBOOK (notebook), NULL));
		  break;

	  case DOC_GOTO_FIRST:
		  gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 0);
		  break;
	  }

}
/*
 * fin de move_notebook_page 
 */

/*
 * returns the new vertical adjustement float value for a given position
 */
float 
position_2_adjustment (GtkWidget * widget, int position)
{

	char *buf, *haystack, *needle;
	int numlines, linenum;
	int a, b, len, i;
	float newadj;

	linenum = 1;
	numlines = 1;

	len = gtk_text_get_length (GTK_TEXT (widget));

	if (len == 0)
	  {
		  return 0;
	  }

	buf = gtk_editable_get_chars (GTK_EDITABLE (widget), 1, len);

	if (!buf)
	  {
		fprintf(stderr,"Couldn't allocate memory !");
		exit(-1);
	  }


	for (i = 0; i < position; i++)
	  {
		  if (buf[i] == '\n')
			  linenum++;
	  }

	a = 1;
	b = len;

	haystack = buf;
	do
	  {
		  needle = strchr (haystack, '\n');
		  if (needle)
			{
				haystack = needle + 1;
				if (linenum == numlines)
					b = needle - buf + 1;
				numlines++;
				if (linenum == numlines)
					a = needle - buf + 1;
			}
	  }
	while (needle != NULL);

	g_free (buf);


	newadj = (linenum * GTK_ADJUSTMENT (GTK_TEXT (widget)->vadj)->upper) / numlines - GTK_ADJUSTMENT (GTK_TEXT (widget)->vadj)->page_increment;

	return newadj;

} /* fin de position_2_adjustment */

void
move_in_document (GtkWidget * widget, int Opcode)
{

	char *buff;
	char *buff2;
	int i;
	float j;

	if (numfileopened == 0)
		return;

	i = gtk_text_get_length (GTK_TEXT (sheet[curfile].text));
	buff = gtk_editable_get_chars (GTK_EDITABLE (sheet[curfile].text), 0, i);

	if (!buff)
	  {
		fprintf(stderr, "Couldn't allocate memory !");
		exit(-1);
	  }

	switch (Opcode)
	  {

	  case DOC_MOVE_BODY:
		  if (strstr (buff, "<BODY") == NULL)
			{
				if (strstr (buff, "<body") == NULL)
				  {
					  merror (_ ("couldn't find the body tag"));
					  g_free(buff);
					  buff=0;
					  return;
				  }
				else
				  {
					  buff2 = strstr (buff, "<body");
					  i = strlen (buff2);
					  j = position_2_adjustment (sheet[curfile].text, gtk_text_get_length (GTK_TEXT (sheet[curfile].text)) - i);
					  gtk_adjustment_set_value (GTK_ADJUSTMENT (GTK_TEXT (sheet[curfile].text)->vadj), j);
					  gtk_editable_set_position (GTK_EDITABLE (sheet[curfile].text), gtk_text_get_length (GTK_TEXT (sheet[curfile].text)) - i);
				  }
			}
		  else
			{
				buff2 = strstr (buff, "<BODY");
				i = strlen (buff2);
				j = position_2_adjustment (sheet[curfile].text, gtk_text_get_length (GTK_TEXT (sheet[curfile].text)) - i);
				gtk_adjustment_set_value (GTK_ADJUSTMENT (GTK_TEXT (sheet[curfile].text)->vadj), j);
				gtk_editable_set_position (GTK_EDITABLE (sheet[curfile].text), gtk_text_get_length (GTK_TEXT (sheet[curfile].text)) - i);
			}
		  break;

	  case DOC_MOVE_HEAD:
		  if (strstr (buff, "<HEAD") == NULL)
			{
				if (strstr (buff, "<head") == NULL)
				  {
					  merror (_ ("couldn't find the head tag"));
					  g_free(buff);
					  buff=0;
					  return;
				  }
				else
				  {
					  buff2 = strstr (buff, "<head");
					  i = strlen (buff2);
					  j = position_2_adjustment (sheet[curfile].text, gtk_text_get_length (GTK_TEXT (sheet[curfile].text)) - i);
					  gtk_adjustment_set_value (GTK_ADJUSTMENT (GTK_TEXT (sheet[curfile].text)->vadj), j);
					  gtk_editable_set_position (GTK_EDITABLE (sheet[curfile].text), gtk_text_get_length (GTK_TEXT (sheet[curfile].text)) - i);
				  }
			}
		  else
			{
				buff2 = strstr (buff, "<HEAD");
				i = strlen (buff2);
				j = position_2_adjustment (sheet[curfile].text, gtk_text_get_length (GTK_TEXT (sheet[curfile].text)) - i);
				gtk_adjustment_set_value (GTK_ADJUSTMENT (GTK_TEXT (sheet[curfile].text)->vadj), j);
				gtk_editable_set_position (GTK_EDITABLE (sheet[curfile].text), gtk_text_get_length (GTK_TEXT (sheet[curfile].text)) - i);
			}
		  break;

	  case DOC_MOVE_BOTTOM:

		  j = position_2_adjustment (sheet[curfile].text, strlen (buff));

		  gtk_adjustment_set_value (GTK_ADJUSTMENT (GTK_TEXT (sheet[curfile].text)->vadj), j);
		  gtk_editable_set_position (GTK_EDITABLE (sheet[curfile].text), gtk_text_get_length (GTK_TEXT (sheet[curfile].text)));

		  break;

	  case DOC_MOVE_TOP:
		  j = position_2_adjustment (sheet[curfile].text, 0);

		  gtk_adjustment_set_value (GTK_ADJUSTMENT (GTK_TEXT (sheet[curfile].text)->vadj), j);
		  gtk_editable_set_position (GTK_EDITABLE (sheet[curfile].text), 0);
		  break;
	  }

	g_free (buff);
	buff = 0;
}
